/*
 Death by Hanging::scripts/Character.js
 Copyright (C) 2014 granony
 This software is released under the MIT License.
 http://opensource.org/licenses/mit-license.php
 
 キャラクタの作成・管理
*/
function Character(params, rm){
  "use strict";
  
  var self = this;
  var params = params;
  var world = params.world;
  var scale = params.scale;  

  var emotionNum   = params.emotionNum   || 0;
  var hairNum      = params.hairNum      || 0;
  var stateNum     = 0;
  var eyesColorNum = params.eyesColorNum || 0;
  var glassesNum   = params.glassesNum   || 0;
  var costumeNum   = params.costumeNum   || 0;
  var ubodyNum     = params.ubodyNum     || 0;
  
  var step = 0;
    
  ////////////////////////////////////////////////////////////////////////
  // 身体の定義
  //////////////////////////////////////////////////////////////////////// 
  // 頭
  this.head = createBody(world, b2Body.b2_staticBody, false, [
      new b2Vec2( 80/scale, 0/scale),
      new b2Vec2( 80/scale, 200/scale),
      new b2Vec2(-80/scale, 200/scale),
      new b2Vec2(-80/scale, 0/scale)
    ], 260/scale, 20/scale, 0
  );
  var headCanvas      = createCanvas(500,500);
  var faceCanvas      = createCanvas(500,500);
  var hairSideCanvas  = createCanvas(500,500);
  var hairFrontCanvas = createCanvas(500,500);
  var hairBackCanvas  = createCanvas(600,1000);
  var eyesCanvas      = createCanvas(500,500);
  var mouthCanvas     = createCanvas(500,500);
  var headCtx      = headCanvas.getContext("2d");
  var faceCtx      = faceCanvas.getContext("2d");
  var hairSideCtx  = hairSideCanvas.getContext("2d");
  var hairFrontCtx = hairFrontCanvas.getContext("2d");
  var hairBackCtx  = hairBackCanvas.getContext("2d");
  var eyesCtx      = eyesCanvas.getContext("2d");
  var mouthCtx     = mouthCanvas.getContext("2d");
  this.hairBackCanvas = hairBackCanvas;
  extendUD(this.head, {name:"head", type:"meat", canvas:headCanvas});
  
  this.ubody = createBody(world, b2Body.b2_staticBody, false, [
      new b2Vec2( 160/scale, 0/scale),
      new b2Vec2( 160/scale, 570/scale),
      new b2Vec2(-160/scale, 570/scale),
      new b2Vec2(-160/scale, 0/scale)
    ], 300/scale, 110/scale, 0, 50
  );
  var ubodyCanvas = createCanvas(600,1200);
  var ubodyCtx = ubodyCanvas.getContext("2d");
  var ubodyBackCanvas = createCanvas(600,1200);
  var ubodyBackCtx = ubodyBackCanvas.getContext("2d");
  this.ubodyBackCanvas = ubodyBackCanvas;
  extendUD(this.ubody, {name:"ubody", type:"meat", canvas:ubodyCanvas});
  
  var headUbody  = createRevoluteJoint(world, this.head, this.ubody, 
    40/scale,  140/scale,
    0/scale,  50/scale, -30, 30);
  
  this.lleg = createBody(world, b2Body.b2_dynamicBody, false, [
      new b2Vec2( 60/scale, 0/scale),
      new b2Vec2( 60/scale, 500/scale),
      new b2Vec2(-60/scale, 500/scale),
      new b2Vec2(-60/scale, 0/scale)
    ], 350/scale, 500/scale, 0
  );
  var llegCanvas = createCanvas(600,800);
  var llegCtx = llegCanvas.getContext("2d");
  extendUD(this.lleg, {name:"lleg", type:"meat", canvas:llegCanvas});
  
  var ubodyLleg  = createRevoluteJoint(world, this.ubody, this.lleg, 
    50/scale,  400/scale,
    0/scale,  60/scale, -30, 30);
  
  this.rleg = createBody(world, b2Body.b2_dynamicBody, false, [
      new b2Vec2( 60/scale, 0/scale),
      new b2Vec2( 60/scale, 500/scale),
      new b2Vec2(-60/scale, 500/scale),
      new b2Vec2(-60/scale, 0/scale)
    ], 220/scale, 500/scale, 0
  );
  var rlegCanvas = createCanvas(600,800);
  var rlegCtx = rlegCanvas.getContext("2d");
  extendUD(this.rleg, {name:"rleg", type:"meat", canvas:rlegCanvas});
  
  var ubodyRleg  = createRevoluteJoint(world, this.ubody, this.rleg, 
    -80/scale,  400/scale,
    0/scale,  60/scale, -30, 30);
  
  this.rope1 = createBody(world, b2Body.b2_dynamicBody, false, [
      new b2Vec2( 10/scale, 0/scale),
      new b2Vec2( 10/scale, 100/scale),
      new b2Vec2(-10/scale, 100/scale),
      new b2Vec2(-10/scale, 0/scale)
    ], 350/scale, 90/scale, 0
  );
  var rope1Canvas = createCanvas(200,200);
  var rope1Ctx = rope1Canvas.getContext("2d");
  extendUD(this.rope1, {name:"rope1", type:"meat", canvas:rope1Canvas});
  var rope1Image = rm.images.system["rope_1"];
  rope1Ctx.drawImage(rope1Image,0,0);
  
  var rope1Ubody  = createRevoluteJoint(world, this.rope1, this.ubody, 
    -10/scale,  90/scale,
    50/scale,  80/scale, -40, 40);
    
  this.rope2 = createBody(world, b2Body.b2_dynamicBody, true, [
      new b2Vec2( 10/scale, 0/scale),
      new b2Vec2( 10/scale, 480/scale),
      new b2Vec2(-10/scale, 480/scale),
      new b2Vec2(-10/scale, 0/scale)
    ], 350/scale, 100/scale, 0
  );
  var rope2Canvas = createCanvas(200,600);
  var rope2Ctx = rope2Canvas.getContext("2d");
  extendUD(this.rope2, {name:"rope2", type:"meat", canvas:rope2Canvas});
  var rope2Image = rm.images.system["rope_2"];
  rope2Ctx.drawImage(rope2Image,0,0);
  
  var rope2Rope1  = createRevoluteJoint(world, this.rope2, this.rope1, 
    0/scale,  460/scale,
    5/scale,  20/scale, -40, 40);
    
  this.wall = createBody(world, b2Body.b2_staticBody, true, [
      new b2Vec2( WIDTH/2/scale, -200/scale),
      new b2Vec2( WIDTH/2/scale, HEIGHT/scale),
      new b2Vec2(-WIDTH/2/scale, HEIGHT/scale),
      new b2Vec2(-WIDTH/2/scale, -200/scale)
    ], 300/scale, -200/scale, 0
  );
  
  var wallRope2  = createRevoluteJoint(world, this.wall, this.rope2, 
    -20/scale,  -120/scale,
    0/scale,  0/scale, -40, 40);
  
  this.displayList = [this.wall, this.rope2, this.rope1, this.lleg, this.rleg, this.ubody, this.head];

  /************************************************************************
   * 描画
   ************************************************************************/
  var updateFace = function(){
    clearCtx(faceCtx);
    var imageID = imageDefs["face"][emotionNum][hairNum][stateNum]; // [emotion][hair][state]
    var image = rm.images.face[imageID];
    faceCtx.drawImage(image,0,0,image.width,image.height,
      (faceCanvas.width-image.width)/2, (faceCanvas.height-image.height)/2,
      image.width, image.height
    );
  }
  
  var updateHair = function(){
    clearCtx(hairSideCtx);
    clearCtx(hairFrontCtx);
    clearCtx(hairBackCtx);
    var hairFrontImage = rm.images.hair[ imageDefs["hair"][hairNum][0] ];
    var hairSideImage  = rm.images.hair[ imageDefs["hair"][hairNum][1] ];
    var hairBackImage  = rm.images.hair[ imageDefs["hair"][hairNum][2] ];
    
    hairFrontCtx.drawImage(hairFrontImage,0,0,hairFrontImage.width,hairFrontImage.height,
      (hairFrontCanvas.width-hairFrontImage.width)/2, (hairFrontCanvas.height-hairFrontImage.height)/2,
      hairFrontImage.width, hairFrontImage.height
    );
    hairSideCtx.drawImage(hairSideImage,0,0,hairSideImage.width,hairSideImage.height,
      (hairSideCanvas.width-hairSideImage.width)/2, (hairSideCanvas.height-hairSideImage.height)/2,
      hairSideImage.width, hairSideImage.height);
    hairBackCtx.drawImage(hairBackImage,0,0, hairBackImage.width,hairBackImage.height,
      (hairBackCanvas.width-hairBackImage.width)/2,0,
      hairBackImage.width, hairBackImage.height
    );
  }
  
  var updateEyes = function(){
    var pattern = eyesPatterns[emotionNum][stateNum];
    var period = 0;
    for(var i=0; i<pattern.length; i++){
      period+=pattern[i];
    }
    var count = 0;
    var patternNum = 0;
    if(pattern[0]!=0){
      for(var i=0; i<pattern.length; i++){
        count = count+pattern[i];
        if(step%period<count){
          patternNum = i;
          break;
        }
      }
    }
    clearCtx(eyesCtx);
    var eyesID = imageDefs["eyes"][emotionNum][eyesColorNum][stateNum][patternNum];
    var glassesID = imageDefs["glasses"][glassesNum];
    var eyesImage = rm.images.eyes[eyesID];
    var glassesImage = rm.images.glasses[glassesID];
    eyesCtx.drawImage(eyesImage,0,0,eyesImage.width,eyesImage.height,
      (eyesCanvas.width-eyesImage.width)/2, (eyesCanvas.height-eyesImage.height)/2,
      eyesImage.width, eyesImage.height
    );
    eyesCtx.drawImage(glassesImage,0,0,glassesImage.width,glassesImage.height,
      (eyesCanvas.width-glassesImage.width)/2, (eyesCanvas.height-glassesImage.height)/2,
      glassesImage.width, glassesImage.height
    );
  }
  
  var updateMouth = function(){
    var pattern = mouthPatterns[emotionNum][stateNum];
    var period = 0;
    for(var i=0; i<pattern.length; i++){
      period+=pattern[i];
    }
    var count = 0;
    var patternNum = 0;
    if(pattern[0]!=0){
      for(var i=0; i<pattern.length; i++){
        count = count+pattern[i];
        if(step%period<count){
          patternNum = i;
          break;
        }
      }
    }
    clearCtx(mouthCtx);
    var imageID = imageDefs["mouth"][emotionNum][stateNum][patternNum];
    var mouthImage = rm.images.mouth[imageID];
    mouthCtx.drawImage(mouthImage,0,0,mouthImage.width,mouthImage.height,
      (mouthCanvas.width-mouthImage.width)/2, (mouthCanvas.height-mouthImage.height)/2,
      mouthImage.width, mouthImage.height
    );
  }
  
  var updateHead = function(){
    clearCtx(headCtx);
    headCtx.drawImage(hairSideCanvas, 0, 0);
    headCtx.drawImage(faceCanvas, 0, 0);
    headCtx.drawImage(mouthCanvas, 0, 0);
    headCtx.drawImage(eyesCanvas, 0, 0);
    headCtx.drawImage(hairFrontCanvas, 0, 0);
  }
  
  var updateUbody = function(){
    clearCtx(ubodyCtx);
    clearCtx(ubodyBackCtx);
    var ubodyImage = rm.images.ubody[ imageDefs["ubody"][ubodyNum][stateNum] ];
    var ubodyCostumeImage = rm.images.costume[ imageDefs["costume"][costumeNum][0] ];
    var ubodyBackCostumeImage = rm.images.costume[ imageDefs["costume"][costumeNum][3] ];
    ubodyCtx.drawImage(ubodyImage,0,0);
    ubodyCtx.drawImage(ubodyCostumeImage,0,0);
    ubodyBackCtx.drawImage(ubodyBackCostumeImage,0,0);
  }

  var updateLleg = function(){
    clearCtx(llegCtx);
    var llegImage = rm.images.lleg[ imageDefs["lleg"][stateNum] ];
    var llegCostumeImage = rm.images.costume[ imageDefs["costume"][costumeNum][1] ];
    llegCtx.drawImage(llegImage,0,0);
    llegCtx.drawImage(llegCostumeImage,0,0);
  }
  
  var updateRleg = function(){
    clearCtx(rlegCtx);
    var rlegImage = rm.images.rleg[ imageDefs["rleg"][stateNum] ];
    var rlegCostumeImage = rm.images.costume[ imageDefs["costume"][costumeNum][2] ];
    rlegCtx.drawImage(rlegImage,0,0);
    rlegCtx.drawImage(rlegCostumeImage,0,0);
  }
  
  this.update = function(){
    step++;
    updateFace();
    updateEyes();
    updateMouth();
    updateHair();
    updateHead(); // 他の頭に関係する更新の終了後に呼ぶ
    updateUbody();
    updateLleg();
    updateRleg();
  }
  
  this.changeState = function(){
    stateNum++;
    if(stateNum>=totalStates){
      stateNum=totalStates-1;
    }
    params.stateNum = stateNum;
    dbgLog("stateNum=",stateNum);
  }
  
  this.changeEyesColor = function(){
    eyesColorNum = (eyesColorNum+1)%imageDefs.eyes[0].length;
    params.eyesColorNum = eyesColorNum;
    dbgLog("eyesColorNum=",eyesColorNum);
  }
  
  this.changeEmotion = function(){
    emotionNum = (emotionNum+1)%imageDefs.eyes.length;
    params.emotionNum = emotionNum;
    dbgLog("emotionNum=",emotionNum);
  }
  
  this.changeHair = function(){
    hairNum = (hairNum+1)%imageDefs.hair.length;
    params.hairNum = hairNum;
    dbgLog("hairNum=",hairNum);
  }
  
  this.changeGlasses = function(){
    glassesNum = (glassesNum+1)%imageDefs.glasses.length;
    params.glassesNum = glassesNum;
    dbgLog("glassesNum=",glassesNum);
  }
  
  this.changeCostume = function(){
    costumeNum = (costumeNum+1)%imageDefs.costume.length;
    params.costumeNum = costumeNum;
    dbgLog("costumeNum=",costumeNum);
  }
  
  this.changeUbody = function(){
    ubodyNum = (ubodyNum+1)%imageDefs.ubody.length;
    params.ubodyNum = ubodyNum;
    dbgLog("costumeNum=",ubodyNum);
  }
}