/*
 Death by Hanging::scripts/PlayScene.js
 Copyright (C) 2014 granony
 This software is released under the MIT License.
 http://opensource.org/licenses/mit-license.php
 
 プレイ用のシーン
*/
function PlayScene(rm, params){
  "use strict";
    
  var dom = jQuery("#game")[0];
  var $canvas = jQuery("#canvas");
  var canvas = $canvas[0];
  var ctx = canvas.getContext("2d");
  var systemImages = rm.images.system;
  var sounds = rm.sounds;
  var animation;
  var strongCnovlusionsFlag;
  var weakConvlusionsFlag;
  var character = params.character;
  var displayList = character.displayList;
  var world = params.world;
  var scale = params.scale;
  var canBreath = true;

  var backgroundCanvas = createCanvas(WIDTH,HEIGHT);
  var backgroundCtx = backgroundCanvas.getContext("2d");
  var backgroundImage = systemImages["background_1"];
  backgroundCtx.drawImage(systemImages["background_1"],0,0);

  ////////////////////////////////////////////////////////////////////////
  // ボタン押下時のハンドラ
  ////////////////////////////////////////////////////////////////////////  
  var onGoButtonClicked = function(){
    buttonGo.destroy();
    buttonGo      = createSystemButton(onRecoverButtonClicked,systemImages["button_recover"], 530,380);
    window.setTimeout(onDoorOpened,8*1000/30);
    sounds["door"].currentTime=0;
    sounds["door"].play();
  }
  
  var onRecoverButtonClicked = function(){
    // 死亡後のリスタートではびっくりした表情でスタートさせたいなぁ
    dom.dispatchEvent(createChangeSceneEvent("title"));
  };
  
  var onEmotionButtonClicked = function(){
    character.changeEmotion();
  }
  
  var onHairButtonClicked = function(){
    character.changeHair();
  }
  
  var onEyesButtonClicked = function(){
    character.changeEyesColor();
  }
  
  var onGlassesButtonClicked = function(){
    character.changeGlasses();
  }
  
  var onCostumeButtonClicked = function(){
    character.changeCostume();
  }
  
  var onBreastButtonClicked = function(){
    character.changeUbody();
  }

  ////////////////////////////////////////////////////////////////////////
  // ボタン
  ////////////////////////////////////////////////////////////////////////  
  var createSystemButton = function(func,image,x,y){
    var canvas1 = createButtonImage(image, 60, 60, "#000000", "#FFFFFF");
    var canvas2 = createButtonImage(image, 60, 60, "#FFFFFF", "#FF0000");
    return createButton(func, canvas1, canvas2, x, y);
  }
  
  var buttonEmotion = createSystemButton(onEmotionButtonClicked,systemImages["button_emotion"],530,10);
  var buttonEyes    = createSystemButton(onEyesButtonClicked,systemImages["button_eyes"],   530,70);
  var buttonHair    = createSystemButton(onHairButtonClicked,systemImages["button_hair"],   530,130);
  var buttonGlasses = createSystemButton(onGlassesButtonClicked,systemImages["button_glasses"],530,190);
  var buttonCostume = createSystemButton(onCostumeButtonClicked,systemImages["button_costume"],530,250);
  var buttonBreast  = createSystemButton(onBreastButtonClicked, systemImages["button_breast"], 530, 310);
  
  var buttonGo      = createSystemButton(onGoButtonClicked,systemImages["button_go"],     530,380);
  var buttonCamera  = createSystemButton(captureScreenShot,systemImages["button_camera"], 530,450);
  
  ////////////////////////////////////////////////////////////////////////
  // stateの移行用ハンドラ
  ////////////////////////////////////////////////////////////////////////  
  var onDoorOpened = function(){
    backgroundCtx.drawImage(systemImages["background_2"],0,0);
    canBreath = false;
    character.head.SetPosition(
      new b2Vec2(headDefaultPositionX, headDefaultPositionY));
    character.ubody.SetPosition(
      new b2Vec2(ubodyDefaultPositionX, ubodyDefaultPositionY));
    character.head.SetType(b2Body.b2_dynamicBody);
    character.ubody.SetType(b2Body.b2_dynamicBody);
    character.ubody.SetLinearVelocity(new b2Vec2(0,1));
    character.changeState();
    window.setTimeout(onCharacterFallen, 15*1000/30);
  };
  
  var onCharacterFallen = function(){
    character.changeState();
    strongCnovlusionsFlag = window.setInterval(convluseStrongly, 2*1000/30);
    window.setTimeout(onCharacterEndured, 180*1000/30);
  }
  
  var onCharacterEndured = function(){
    character.changeState();
    clearInterval(strongCnovlusionsFlag);
    weakConvlusionsFlag = window.setInterval(convluseWeakly, 2*1000/30);
    window.setTimeout(onCharacterDied, 240*1000/30);
  }
  
  var onCharacterDied = function(){
    character.changeState();
    clearInterval(weakConvlusionsFlag);
    peeCount = 0;
    setTimeout(emergePee,30*1000/30);
  }
  
  var convluseStrongly = function(){
    var rand1 = 1000*(Math.random()-0.5);
    var rand2 = 1000*(Math.random()-0.5);
    var rand3 = 50*(Math.random()-0.5);
    character.lleg.ApplyImpulse(new b2Vec2(rand1,0), new b2Vec2(0,400/scale));
    character.rleg.ApplyImpulse(new b2Vec2(rand2,0), new b2Vec2(0,400/scale));
    character.head.ApplyImpulse(new b2Vec2(rand3,0), new b2Vec2(0,0));
  }
  
  var convluseWeakly = function(){
    var rand1 = 300*(Math.random()-0.5);
    var rand2 = 300*(Math.random()-0.5);
    var rand3 = 20*(Math.random()-0.5);
    character.lleg.ApplyImpulse(new b2Vec2(rand1,0), new b2Vec2(0,400/scale));
    character.rleg.ApplyImpulse(new b2Vec2(rand2,0), new b2Vec2(0,400/scale));
    character.head.ApplyImpulse(new b2Vec2(rand3,0), new b2Vec2(0,0));
  }
  

  ////////////////////////////////////////////////////////////////////////
  // おしっこ関係
  ////////////////////////////////////////////////////////////////////////  
  var peeCount;
  var peeList = [];
  
  // 初期に水滴用のb2Bodyをすべて作成しておく
  // b2Body の作成コストは結構大きい
  (function(){
    for(var i=0; i<120; i++){
      var rand = Math.random();
      var image;
      if(rand<1/3){
        image = rm.images.pee["pee_1"];
      }else if(rand<2/3){
        image = rm.images.pee["pee_2"];
      }else{
        image = rm.images.pee["pee_3"];
      }
      
      var peeBody = createBody(world, b2Body.b2_staticBody, true, [
          new b2Vec2( 20/scale, 0/scale),
          new b2Vec2( 20/scale, 20/scale),
          new b2Vec2(-20/scale, 20/scale),
          new b2Vec2(-20/scale, 0/scale)
        ], -100,-100,0
      );
      var peeCanvas = createCanvas(image);
      peeList.push([peeBody, peeCanvas]);
    }
  })();

  // 時期がきたら水滴用 b2Body を表示
  // 初期位置を修正して，物理演算の対象とする
  var emergePee = function(){
    var pee = peeList[peeCount];
    var body = pee[0];
    var bang = character.ubody.GetAngle();
    var x = character.ubody.GetPosition().x-440/scale*fastSin(bang)-15/scale*fastCos(bang);
    var y = character.ubody.GetPosition().y+440/scale*fastCos(bang);
    body.SetType(b2Body.b2_dynamicBody);
    body.SetPosition(new b2Vec2(x,y));
    body.ApplyImpulse(new b2Vec2(Math.random(),0), new b2Vec2(0, 10/scale));
    peeCount++;
    if(peeCount<peeList.length){
      setTimeout(emergePee, 1000/30);
    }
  }
  
  ////////////////////////////////////////////////////////////////////////
  // クレジット
  ////////////////////////////////////////////////////////////////////////
  var creditCanvas1 = createTextImage(
    "Death by Hanging "+VERSION, 
     140, 16, "normal bold 12px sans-serif", "#FFFFFF", "left");
  var creditCanvas2 = createTextImage(
    "@ granony ", 
     140, 16, "normal bold 12px sans-serif", "#FFFFFF", "right");
  
  ////////////////////////////////////////////////////////////////////////
  // 描画
  ////////////////////////////////////////////////////////////////////////
  // 後ろ髪をメインのキャンバスに描画
  var head = character.head;
  var drawBackHair = function(){
    var bx = head.GetPosition().x*scale;
    var by = head.GetPosition().y*scale;
    var bang = head.GetAngle();
    var sCanvas = character.hairBackCanvas;
    var headData = head.GetUserData();
    ctx.save();
    ctx.translate(bx,by);
    ctx.rotate(bang/1.5);
    ctx.drawImage(sCanvas, 0, 0, sCanvas.width, sCanvas.height,
      -sCanvas.width/2+50*fastSin(-bang/1.5), -(headData.canvas.height-headData.height)/2,
      sCanvas.width, sCanvas.height);
    ctx.restore();
  }
  
  // 背中側のコスチュームをメインのキャンバスに描画
  var ubody = character.ubody;
  var drawBackCostume = function(){
    var bx = ubody.GetPosition().x*scale;
    var by = ubody.GetPosition().y*scale;
    var bang = ubody.GetAngle();
    var sCanvas = character.ubodyBackCanvas;
    var ud = ubody.GetUserData();
    ctx.save();
    ctx.translate(bx,by);
    ctx.rotate(bang/1.5);
    ctx.drawImage(sCanvas, 0, 0, sCanvas.width, sCanvas.height,
      -sCanvas.width/2, -(ud.canvas.height-ud.height)/2,
      sCanvas.width, sCanvas.height);
    ctx.restore();
  }
  
  // body を一つメインのキャンバスに描画
  var drawBody = function(b){
    var bx = b.GetPosition().x*scale;
    var by = b.GetPosition().y*scale;
    var bang = b.GetAngle();
    
    var ud=b.GetUserData();

    if(ud && ud.canvas){
      ctx.save();
      ctx.translate(bx,by);
      ctx.rotate(bang);
      ctx.drawImage(
        ud.canvas,0,0,ud.canvas.width,ud.canvas.height,
                  -ud.canvas.width/2,-(ud.canvas.height-ud.height)/2,
                  ud.canvas.width, ud.canvas.height
      );
      ctx.restore();
    }
    //console.log(ud.name,bx,by)
    // デバッグ時には b2Body の輪郭を描画する
    if(DEBUG){
      ctx.save();
      ctx.translate(bx,by);
      ctx.rotate(bang);
      for (var f = b.GetFixtureList(); f; f= f.GetNext()){
        var s= f.GetShape();
        
        if(s.GetType()==b2Shape.e_polygonShape){
          var vs = s.GetVertices();
          var x = vs[0].x*scale;
          var y = vs[0].y*scale;
          ctx.lineWidth = 2;
          ctx.strokeStyle = "#000000";
          ctx.beginPath();
          ctx.moveTo(x,y);
          for(var i=1; i<vs.length; i++){ // s.GetVertexCount でもいいかも
            x = vs[i].x*scale;
            y = vs[i].y*scale;
            ctx.lineTo(x,y);
          }
          ctx.closePath();
          ctx.stroke();
        }
      }
      ctx.restore(); 
    }
  };
  
  // 各 body の描画ルーチンを順に呼び出す 
  var drawBox2D = function(){    
    for(var i=0, iLength=displayList.length; i<iLength; i++){
      drawBody(displayList[i]);
    }

    // デバッグ用の joint の描画
    if(DEBUG){
    for (var j = world.GetJointList(); j; j = j.GetNext()) {
        var b1 = j.GetBodyA();
        var b2 = j.GetBodyB();
        var b1x = b1.GetPosition().x*scale;
        var b1y = b1.GetPosition().y*scale;
        var b2x = b2.GetPosition().x*scale;
        var b2y = b2.GetPosition().y*scale;
        var x1 = j.GetAnchorA().x*scale;
        var y1 = j.GetAnchorA().y*scale;
        var x2 = j.GetAnchorB().x*scale;
        var y2 = j.GetAnchorB().y*scale;
        ctx.strokeStyle = '#FF0000';
        ctx.beginPath();
        ctx.moveTo(b1x,b1y);
        ctx.lineTo(x1,y1);
        ctx.stroke();
        ctx.strokeStyle = '#0000FF';
        ctx.beginPath();
        ctx.moveTo(b2x,b2y);
        ctx.lineTo(x2,y2);
        ctx.stroke();
      }
    }
  };
  
  
  // 座標の参考用にグリッドラインを引く デバッグ用
  function drawGrid(){
    ctx.strokeStyle = "#CCCCCC";
    for(var i=1; i<=6; i++){
      ctx.beginPath();
      ctx.moveTo(0,i*100);
      ctx.lineTo(WIDTH,i*100);
      ctx.stroke();
    }
    for(var i=1; i<=8; i++){
      ctx.beginPath();
      ctx.moveTo(i*100,0);
      ctx.lineTo(i*100,HEIGHT);
      ctx.stroke();
    }
  }
  
  // 背景をメインのキャンバスに描画
  var drawBackground = function(){
    ctx.drawImage(backgroundCanvas,0,0);
  }
  
  // ボタンとクレジットの描画
  var drawControler = function(){
    buttonEmotion.draw();
    buttonEyes.draw();
    buttonHair.draw();
    buttonGlasses.draw();
    buttonCostume.draw();
    buttonBreast.draw();
    buttonGo.draw();
    //buttonRecover.draw();
    buttonCamera.draw();
    ctx.drawImage(creditCanvas1, 460, 860);
    ctx.drawImage(creditCanvas2, 460, 880);
  }
  
  // おしっこの描画
  var drawPee = function(){
    for(var i=0; i<peeList.length; i++){
      var body = peeList[i][0];
      var canvas = peeList[i][1];
      var bx = body.GetPosition().x*scale;
      var by = body.GetPosition().y*scale;
      var bang = body.GetAngle();

      ctx.save();
      ctx.translate(bx,by);
      ctx.rotate(bang);
      ctx.drawImage(canvas,0,0,canvas.width,canvas.height,
        -canvas.width/2,0,
        canvas.width,canvas.height
      );
      ctx.restore();
    }
  }
 
  ////////////////////////////////////////////////////////////////////////
  // 呼吸
  ////////////////////////////////////////////////////////////////////////
  var headDefaultPositionX;
  var headDefaultPositionY;
  var ubodyDefaultPositionX;
  var ubodyDefaultPositionY;
  var breathStep = 0;
  var breathStep2 =0;
  // 呼吸
  var doBreath = function(){
    if(breathStep2>=0){
      breathStep++
    }else{
      breathStep2++;
      if(breathStep2==0){breathStep++};
    }
    if((breathStep+60)%120==0 && breathStep2>=0){
      breathStep2 =-10;
    }
    if(canBreath){
    var dy = (-2-2*Math.cos(breathStep/120*2*Math.PI))/scale;
      character.head.SetPosition(
        new b2Vec2(headDefaultPositionX, headDefaultPositionY+dy*0.5));
      character.ubody.SetPosition(
        new b2Vec2(ubodyDefaultPositionX, ubodyDefaultPositionY+dy));
    }
  }
  
  ////////////////////////////////////////////////////////////////////////
  // 更新
  ////////////////////////////////////////////////////////////////////////
  function update() {
    doBreath();
    world.Step(1/30, 20, 10);
    character.update();
    drawBackground();
    drawBackHair();
    drawBackCostume();
    drawPee();
    drawBox2D();
    drawControler();
    
  };
  
  ////////////////////////////////////////////////////////////////////////
  // シーンオブジェクト
  ////////////////////////////////////////////////////////////////////////
  var start = function(){
    // 接続の不整合を解消するため，30ステップ進める
    for(var i=0; i<30; i++){
      world.Step(1/30, 20, 10); //1/30, 20, 10
    }
    headDefaultPositionX  = character.head.GetPosition().x;
    headDefaultPositionY  = character.head.GetPosition().y;
    ubodyDefaultPositionX = character.ubody.GetPosition().x;
    ubodyDefaultPositionY = character.ubody.GetPosition().y;
    
    animation = window.setInterval(update, 1000/30);
  };
  
  return {
    start: start,
    stop: function(){
      // アニメーションの停止
      clearInterval(animation);
      // ボディの廃棄
      for(var i=0, iLength=displayList.length; i<iLength; i++){
          world.DestroyBody(displayList[i]);
      }
      for(var i=0; i<peeList.length; i++){
        world.DestroyBody(peeList[i][0]);
      }
      // ボタンの廃棄
      buttonEmotion.destroy();
      buttonEyes.destroy();
      buttonHair.destroy();
      buttonGlasses.destroy();
      buttonCostume.destroy();
      buttonBreast.destroy();
      buttonGo.destroy();
      buttonCamera.destroy();
    },
  }
}
