/*
 Death by Hanging::scripts/ResourceManager.js
 Copyright (C) 2014 granony
 This software is released under the MIT License.
 http://opensource.org/licenses/mit-license.php
 
 外部リソースの管理を行う
*/
function ResourceManager(){
  var dom = jQuery("#game")[0];
  var self = this;

  // 再生可能な音楽形式
  var canPlayOgg;
  var canPlayMP3;
  
  //// リソースの保管用
  // 画像 
  this.images = {};
  // 音
  this.sounds  = {};
  
  //// リソースの個数管理
  // リソース数の合計
  this.totalResourceNum = 0; // = imageList.length + soundDefs.length;
  // ロードされたリソース数
  var loadedImageNum = 0;
  var loadedSoundNum = 0;
  this.loadedResourceNum = 0; // 進行状況の確認に使われる
  
  // 画像定義の線形な配列
  var imageList = [];
  // 音楽定義の線形な配列
  var soundList = [];
  
  // 初期化
  var init = function(){
    // 再生可能な音楽形式を取得
    var audio = new Audio();
    canPlayOgg = ("" != audio.canPlayType("audio/ogg"));
    canPlayMP3 = ("" != audio.canPlayType("audio/mpeg"));
    
    // 画像定義の線形な配列と格納用配列を作成
    for(var imageType in imageDefs){
      self.images[imageType] = new Array();
      if(   imageType=="lleg"    || imageType=="rleg" 
         || imageType=="system" || imageType=="glasses" || imageType=="pee"){
        for(var i=0; i<imageDefs[imageType].length; i++){
          imageList.push([ imageType, imageDefs[imageType][i] ]);
        }
      }else if(imageType=="hair" || imageType=="costume" || imageType=="ubody"){
        for(var i=0; i<imageDefs[imageType].length; i++){
          for(var j=0; j<imageDefs[imageType][i].length; j++){
            imageList.push([ imageType, imageDefs[imageType][i][j] ]);
          }
        }
      }else if(imageType=="face" || imageType=="mouth"){
        for(var i=0; i<imageDefs[imageType].length; i++){
          for(var j=0; j<imageDefs[imageType][i].length; j++){
            for(var k=0; k<imageDefs[imageType][i][j].length; k++){
              imageList.push([ imageType, imageDefs[imageType][i][j][k] ]);
            }
          }
        }
      }else if(imageType=="eyes"){
        for(var i=0; i<imageDefs[imageType].length; i++){
          for(var j=0; j<imageDefs[imageType][i].length; j++){
            for(var k=0; k<imageDefs[imageType][i][j].length; k++){
              for(var l=0; l<imageDefs[imageType][i][j][k].length; l++){
                imageList.push([ imageType, imageDefs[imageType][i][j][k][l] ]);
              }
            }
          }
        }
      }
    }
  
    // 音楽定義の線形な配列を作成
    for(var soundType in soundDefs){
      for(var i=0; i<soundDefs.length; i++){
        soundList.push(soundDefs[i]);
      }
    }
    
    self.totalResourceNum = imageList.length + soundList.length;
  };
  
  // ダウンロードが進行したことを知らせるイベントを作成
  // LoadScene が受け取る
  var createLoadResourcesProgressEvent = function(){
    var e = document.createEvent("Event");
    e.initEvent("load_resources_progress", true, true);
    return e;
  }
  
  // ダウンロードが全て完了したことを知らせるイベントを作成
  // LoadScene が受け取る
  var createLoadResourcesCompleteEvent=function(){
    var e = document.createEvent("Event");
    e.initEvent("load_resources_complete", true, true);
    return e;
  }
  
  // 画像読み込み進展時のハンドラ
  var loadImageHandler = function(){
    loadedImageNum++;
    self.loadedResourceNum++;
    dom.dispatchEvent(createLoadResourcesProgressEvent());
    if(loadedImageNum >= imageList.length){
      loadSounds(0);
      //dom.dispatchEvent(createLoadResourcesCompleteEvent());
    }else{
      loadImages(loadedImageNum);
    }
  };
  
  // 音楽読み込み進展時のハンドラ
  var loadSoundHandler = function(){
    loadedSoundNum++;
    self.loadedResourceNum++;
    dom.dispatchEvent(createLoadResourcesProgressEvent());
    if(loadedSoundNum >= soundList.length){
      dom.dispatchEvent(createLoadResourcesCompleteEvent());
    }else{
      loadSounds(loadedSoundNum);
    }
    this.removeEventListener("canplaythrough", loadSoundHandler);
  };
  
  
  // 画像ファイルを順番に読み込む
  var loadImages = function(idx){
    var imageType = imageList[idx][0];
    var image = new Image();
    image.addEventListener("load",loadImageHandler);
    image.onerror = function(){
    alert( image.src + "の読み込みに失敗しました．\n"
            +"アップデートの作業中かもしれません．\n"
            +"しばらく待った後に再読み込みしても同じエラーが出る場合は，管理人にご連絡ください．");
    }
    var src = "";
    var resourceName = imageList[idx][1];
    self.images[imageType][resourceName] = image;
    src = "images/"+resourceName+".png";
    dbgLog("loadedImageNum =",loadedImageNum, src);
    image.src = src;
  };
  
  // 音楽ファイルを順番に読み込む
  var loadSounds = function(i){
    var soundType = soundList[i][0];
    var sound = new Audio();
    sound.addEventListener("canplaythrough", loadSoundHandler);
    sound.onerror = function(){
      alert( sound.src + "の読み込みに失敗しました．\n"
            +"アップデートの作業中かもしれません．\n"
            +"しばらく待った後に再読み込みしても同じエラーが出る場合は，管理人にご連絡ください．");
    }
    var resourceName = soundList[i];
    self.sounds[resourceName] = sound;
    var src = "sounds/"+resourceName;
    src +=  canPlayOgg?".ogg":".mp3";
    dbgLog("loadedSoundNum =",loadedSoundNum, src);
    sound.src = src;
  };
    
  // ロードの開始
  this.load = function(){
    init();  
    loadImages(0);
  };
};
