/*
 Death by Hanging::scripts/Util.js
 Copyright (C) 2014 granony
 This software is released under the MIT License.
 http://opensource.org/licenses/mit-license.php
 
 グローバルな関数を提供
*/

// おまじない
jQuery.noConflict();

// IE 用の setTimeout, setInterval の拡張
if (document.all && !window.setTimeout.isPolyfill) {
  var __nativeST__ = window.setTimeout;
  window.setTimeout = function (vCallback, nDelay /*, argumentToPass1, argumentToPass2, etc. */) {
    var aArgs = Array.prototype.slice.call(arguments, 2);
    return __nativeST__(vCallback instanceof Function ? function () {
      vCallback.apply(null, aArgs);
    } : vCallback, nDelay);
  };
  window.setTimeout.isPolyfill = true;
}

if (document.all && !window.setInterval.isPolyfill) {
  var __nativeSI__ = window.setInterval;
  window.setInterval = function (vCallback, nDelay /*, argumentToPass1, argumentToPass2, etc. */) {
    var aArgs = Array.prototype.slice.call(arguments, 2);
    return __nativeSI__(vCallback instanceof Function ? function () {
      vCallback.apply(null, aArgs);
    } : vCallback, nDelay);
  };
  window.setInterval.isPolyfill = true;
}

// b2系関数へのエイリアス
var b2Vec2         = Box2D.Common.Math.b2Vec2;
var b2BodyDef      = Box2D.Dynamics.b2BodyDef;
var b2Body         = Box2D.Dynamics.b2Body;
var b2FixtureDef   = Box2D.Dynamics.b2FixtureDef;
var b2Fixture      = Box2D.Dynamics.b2Fixture;
var b2World        = Box2D.Dynamics.b2World;
var b2MassData     = Box2D.Collision.Shapes.b2MassData;
var b2PolygonShape = Box2D.Collision.Shapes.b2PolygonShape;
var b2CircleShape  = Box2D.Collision.Shapes.b2CircleShape;
var b2DebugDraw    = Box2D.Dynamics.b2DebugDraw;
var b2Shape        = Box2D.Collision.Shapes.b2Shape;
var b2RevoluteJointDef = Box2D.Dynamics.Joints.b2RevoluteJointDef;

// 高速な三角関数
var sinTable = (function(){
  var table = [];
  var ang=0;
  angStep = (Math.PI*2)/4096;
  do{
    table.push(Math.sin(ang));
    ang += angStep;
  } while(ang<Math.PI*2);
  return table;
})();

var cosTable = (function(){
  var table = [];
  var ang=0;
  angStep = (Math.PI*2)/4096;
  do{
    table.push(Math.cos(ang));
    ang += angStep;
  } while(ang<Math.PI*2);
  return table;
})();

var myFloor = Math.floor; // 名前解決の手間をはぶく
var myPIInv = 1/Math.PI;

var fastSin = function(ang){
  return sinTable[myFloor(4096*100+4096*ang*myPIInv*0.5)&4095];
}
var fastCos = function(ang){
  return cosTable[myFloor(4096*100+4096*ang*myPIInv*0.5)&4095];
}

// シーンを変更するためのイベントを作成
function createChangeSceneEvent(id){
  var e = document.createEvent("Event");
  e.initEvent("change_scene", true, true);
  e.id = id;
  return e;
}


// 普通の画像ボタンを作成する
function createButtonImage(image, w, h, c1, c2){
  var canvas = createCanvas(w, h);
  var ctx = canvas.getContext("2d");
  if(c1){
    ctx.fillStyle = c1;
    ctx.fillRect(0,0,w,h);
  }
  ctx.drawImage(image,0,0,image.width,image.height,0,0,w,h);
  if(c2){
    ctx.strokeStyle = c2;
    ctx.lineWidth = 2;
    ctx.moveTo(0.5,0.5);
    ctx.lineTo(w-0.5,0.5);
    ctx.lineTo(w-0.5,h-0.5);
    ctx.lineTo(0.5,h-0.5);
    ctx.lineTo(0.5,0.5)
    ctx.stroke();
  }
  return canvas;
};

// テキストボタンの画像を作成する
function createTextButtonImage(text,w,h,c1,c2,c3){
  var canvas = createCanvas(w,h);
  var ctx = canvas.getContext("2d");
  ctx.font = "italic bold "+ h*0.8 +"px sans-serif";
  ctx.textAlign = "center";
  ctx.textBaseline = "top";
  ctx.fillStyle = c2;
  ctx.fillRect(0,0,250,50);
  ctx.strokeStyle = c3;
  ctx.lineWidth = h*0.1;
  ctx.beginPath();
  ctx.moveTo(0,h);
  ctx.lineTo(w,h);
  ctx.lineTo(w,0);
  ctx.stroke()
  ctx.fillStyle = c1;
  ctx.fillText(text,w/2,h*0.1);
  return canvas;
}

// 普通のボタン
// 状態を保持せず，押されたときにハンドラが起動するだけ
function createButton(handler,image1,image2, x, y){
  var $canvas = jQuery("#canvas");
  var canvas = $canvas[0];
  var ctx = canvas.getContext("2d");
  
  var width  = image1.width;
  var height = image1.height;
  var myCanvas = createCanvas(width, height);
  var myCtx = myCanvas.getContext("2d");
  myCtx.drawImage(image1,0,0);
  
  var onClickHandler = function(e){
    var cOffset = $canvas.offset;
    var mx = e.pageX-$canvas.offset().left;
    var my = e.pageY-$canvas.offset().top;
    if(mx>=x && mx<=x+width && my>=y && my<=y+height){
      dbgLog("a button is clicked");
      handler();
    }
  };
  
  var onMouseMoveHandler = function(e){
    var cOffset = $canvas.offset;
    var mx = e.pageX-$canvas.offset().left;
    var my = e.pageY-$canvas.offset().top;
    clearCtx(myCtx);
    if(mx>=x && mx<=x+width && my>=y && my<=y+height){
      myCtx.drawImage(image2,0,0)
    }else{
      myCtx.drawImage(image1,0,0);
    }
  };
  
  $canvas.click(onClickHandler);
  $canvas.mousemove(onMouseMoveHandler)
  
  return {
    draw: function(){
      ctx.drawImage(myCanvas,x,y);
    },
    destroy: function(){
      $canvas.unbind("click", onClickHandler);
      $canvas.unbind("mousemove", onMouseMoveHandler);
    },
  };
}

// テキストで構成された画像を作成する
function createTextImage(text,w,h,font,col,align){
  var canvas = createCanvas(w,h);
  var ctx = canvas.getContext("2d");
  ctx.font = font;
  ctx.fillStyle = col;
  ctx.textBaseline = "top";
  ctx.textAlign = align;
  var x = 0;
  switch(align){
    case("center"): x=w/2; break;
    case("left")  : x=0;   break;
    case("right") : x=w;   break;
  }
  ctx.fillText(text, x, 0);
  return canvas;
};


// b2body を作成する
function createBody(world,type, noCollision, vertices, posx, posy, ang, density){
  var fd = new b2FixtureDef;
  fd.density = 10.0;
  fd.friction = 0.1;
  fd.restitution = 0.01;
  fd.shape = new b2PolygonShape();
  fd.shape.SetAsArray(vertices, vertices.length);
  if(density!=undefined){
    fd.density = density
  }
  if(noCollision){
    fd.filter.categoryBits = 0;
  }
  
  var ud = {};
  var minx=Infinity, maxx=-Infinity;
  var miny=Infinity, maxy=-Infinity;
  for(var i=0; i<vertices.length; i++){
    var x = vertices[i].x;
    var y = vertices[i].y;
    if(x<minx) minx = x;
    if(x>maxx) maxx = x;
    if(y<miny) miny = y;
    if(y>maxy) maxy = y;
  }
  ud.width  = (maxx-minx)*SCALE;
  ud.height = (maxy-miny)*SCALE;
  ud.top    = miny*SCALE;
  ud.bottom = maxy*SCALE;
  

  var bd = new b2BodyDef();
  bd.type = type;
  bd.position.x = posx;
  bd.position.y = posy;
  bd.linearDamping = 0.5;
  bd.angularDamping = 0.5;
  if(ang!=undefined){
    bd.angle = ang;
  }
  var body = world.CreateBody(bd);
  body.CreateFixture(fd);
  body.SetLinearDamping(0.5);
  body.SetAngularDamping(10.0);
  body.SetUserData(ud);
  return body;
};

// revolute joint を作成する
function createRevoluteJoint(world,bA, bB, ax,ay,bx,by, ang1, ang2){
  var jd = new b2RevoluteJointDef();
  jd.Initialize(bA, bB, bA.GetWorldCenter());
  jd.localAnchorA = new b2Vec2(ax, ay);
  jd.localAnchorB = new b2Vec2(bx, by);
  var j =  world.CreateJoint(jd);
  if(ang1!==undefined){
    var flimit = function(A,B){
      var aAng = A.GetAngle()/Math.PI*180;
      var bAng = B.GetAngle()/Math.PI*180;
      return [(ang1+aAng-bAng)/180*Math.PI, (ang2+aAng-bAng)/180*Math.PI];
    };
    j.SetUserData({flimit:flimit, type:"revolute"});
    j.EnableLimit(true);
    j.SetLimits(flimit(bA, bB)[0], flimit(bA, bB)[1]);
  }else{
    j.SetUserData({type:"revolute"});
  }
  return j;
};

// b2Body のユーザーデータに追加する
function extendUD(b, obj){
  var ud = b.GetUserData();
  ud = jQuery.extend(ud, obj);
  b.SetUserData(ud);
}

function createCanvas(a1, a2){
  var canvas = document.createElement("canvas");
  if(a1.width){
    canvas.setAttribute("width", Math.ceil(a1.width));
    canvas.setAttribute("height", Math.ceil(a1.height));
    canvas.getContext("2d").drawImage(a1, 0, 0);
  }else{
    w = a1;
    h = a2;  
    canvas.setAttribute("width", Math.ceil(w));
    canvas.setAttribute("height", Math.ceil(h));
  }
  return canvas;
};

function clearCtx(ctx){
  ctx.clearRect(0,0,WIDTH*2,HEIGHT*2);
}


// コンソールに b2Body 配列の各成分の名前を表意する デバッグ用
function checkBs(bs){
  for(var i=0; i<bs.length; i++){
    console.log(bs[i].GetUserData().name);
  }
}

// スクリーンショットを作成する
var captureScreenShot = function(){
  var $canvas = jQuery("#canvas");
  var canvas = $canvas[0];
  var imageData = canvas.toDataURL("image/png");
  var nwin = window.open();
  nwin.document.write("<!DOCTYPE html>");
  nwin.document.write("<html>");
  nwin.document.write("  <head>");
  nwin.document.write("    <title>Death by Hanging :: Captured Image @ granony's Laboratory</title>");
  nwin.document.write("  </head>");
  nwin.document.write("  <body>");
  nwin.document.write("    <p><img src=\""+imageData+"\" alt=\"\"></p>");
  nwin.document.write("  </body>");
  nwin.document.write("</html>");
};

// DEBUGが有効な場合，任意の数の引数を console に出力する
function dbgLog(){
  if(!DEBUG){return;}
  var msg="";
  for (var i=0; i<arguments.length; i++) {
    var subMsg = arguments[i];
    if(subMsg===null){subMsg="null"}
    if(subMsg===undefined){submsg="undefined"}
    msg+=" "+arguments[i].toString();
  }
  console.log(msg);
}

// 何もしない
var nullFunc = function(){};
  